{
  EWTEXT.PAS -- DLL to display data from INI-structured file in embedded window
Freeware release.
Time-limiting lines commencing with * have been disabled.
}
{$I-}
Library EWTEXT;

Uses WinTypes, WinProcs, WinDos, Strings, HelpDLL;
{$D Copyright  1997 John Carroll jcarroll@pcug.org.au}
{$R EWTEXTFR}

{ Global variables used by the embedded window functions }
Var
  HelpFileName : Array [0..255] of Char;    {needed to derive DataFileName}
  AuthorText : string;      {variables passed to ewDat from helpfile}
  DataFileName : Array [0..255] of Char;    {if not specified, default is derived}
  ParamList : Array [0..24,0..255] of Char; {First dimension must match MaxParam}
  S : string;                         {general-purpose variable}
  counter : integer;                  {general-purpose For..To counter}
  Nsect, Nkey : PChar;                {to get INI values}
  ParamNo : integer;                  {number of data parameters read}
  Allkeys : Boolean;                  {are keynames to be shown?}

Const
  MaxParam: integer = 24;

{
  Timeout -- for evaluation version.
}
{*Procedure Timeout;                                           }
{*var                                                          }
{*y, m, d, dow:         Word;        {for eval version only}   
                                                                
{*Begin
{*GetDate(y,m,d,dow);                                          }
{*if ((y > 1997) or (m > 7)) then                              }
{*MessageBox(0, 'Evaluation period ended.', 'Es finito',       
              MB_OK + MB_ICONSTOP + MB_TASKMODAL);             }
{*End;                                                         }

{ Max -- return the maximum of two words }
Function Max (a, b : Word) : Word;
Begin
  If a < b Then
    Max := b
  Else
    Max := a;
End;

{
  DrawWindowText -- Draw the window text on the specified
  display context.
}
Procedure DrawWindowText (dc : HDC);
Var
  tm : TTextMetric;
Begin
  GetTextMetrics (dc, tm);

  { output parameters extracted from AuthorData }
  for counter:=1 to ParamNo do
    TextOut (dc, tm.tmMaxCharWidth, tm.tmHeight*counter,
             ParamList[counter], StrLen (ParamList[counter]));
End;


{
  GetINIdata -- Read data from an INI file.
  Mods - replace existing string-building in embedded window example;
  take a multi-word parameter as AuthorText & parse into File, Section & Keywords;
  call this repeatedly for each keyword & build up a string,
  adjusting window size to suit.
}
Function GetINIdata : PChar;
var
  INIdata : Array [0..255] of Char;
  A : Array [0..255] of Char;

Begin
  GetPrivateProfileString(Nsect, Nkey, '', INIdata, 255, DataFileName);
  if Allkeys then
    begin
      StrCopy(A,Nkey);
      StrCat(A,'=');
      StrCat(A,INIdata);
      GetINIdata := A;
    end
  else if INIdata[0] = #0 then
    Begin
      StrCopy(INIdata,'Nothing recorded for key "');
      StrCat(INIdata,Nkey);
      StrCat(INIdata,'"');
      GetINIdata := INIdata;
    End
  else
    GetINIdata := INIdata;
End;

{
  Delete leading spaces
}
Function LTrim(Var S : string): string;
begin
  While (Length(S)>1) and (pos(' ',S) = 1) do
  begin
    S := copy(S,2,length(S)-1);
  end;
  if S = ' ' then S := '';
  LTrim:=S;
end;

{
  BlankAll -- clear data parameter array.
}
Procedure BlankAll;

Begin
  for counter := 0 to MaxParam do
    StrCopy(ParamList[counter],'');
  ParamNo := 0;
End;

{ ewDatProc -- Embedded window procedure }
Function ewDatProc (
                    hWindow : hWnd;
                    wMsg : Word;
                    wParam : Word;
                    lParam : Longint) : Longint; Export;
Var
  hfont : THandle;
  tm : TTextMetric;
  ew : pewData;

  ps : TPaintStruct;

  rc : TRect;
  RenderInfo : pRenderInfo;
  pt : TPoint;
  hbm, hbmDefault : hBitmap;
  hbr : HBrush;
  dc : HDC;

  gh : THandle;
  sz : PChar;

  dwExt : Longint;
  lp : PPoint;
  CharPos : Integer;

  Dir: array[0..2, 0..fsDirectory] of Char;
  Name: array[0..2, 0..fsFileName] of Char;
  Ext: array[0..2, 0..fsExtension] of Char;

var
  Separator : Char;

Label
  NoParams;

Begin  
  Case wMsg of
    WM_CREATE :    { Create the window }
    Begin
      ew := pewData (PCreateStruct (lParam)^.lpCreateParams);
      BlankAll;                                       
      Allkeys := False;

      { Save the WM_CREATE information passed by WinHelp }
      StrCopy (HelpFileName, ew^.szFileName);
      if StrPas(ew^.szAuthorData) = '' then
        begin
          StrCopy(ParamList[1],'Author error - no data parameters!!');       
          ParamNo := 1;                                                       
          goto NoParams;
        end;
      AuthorText := StrPas(ew^.szAuthorData);
      ParamNo := 0;
      CharPos := 0;
      For counter := 1 to Length(AuthorText) do
        begin
        Separator := '|';
          if AuthorText[counter] = Separator then {end of parameter}
            begin
              ParamList[ParamNo,CharPos] := #0;   {terminate the string}
              Inc(ParamNo);                       {ready for next parameter...}
              CharPos := 0;                       {...its leading character}
            end
          else        {looking at a character in a parameter}
            begin
              ParamList[ParamNo,CharPos] := AuthorText[counter];
              Inc(Charpos);
            end;
{needed?            if ParamNo = MaxParam then counter := Length(ParamList[0]);   }
        end;
      ParamList[ParamNo,CharPos] := #0;           {terminate final parameter}
      for counter := 0 to ParamNo do              {remove leading spaces}
        begin
            S := StrPas(ParamList[counter]);
            S := LTrim(S);
            StrPCopy(ParamList[counter],S);
        end;

{handle filename - parse for path with FileSplit:
   1 = helpfile directory & name
   2 = sent data or blank
   Sent             Dir     Name     Ext
   ''               2       1        .DAT
   .                1       1        .DAT
   FRED.TXT         2       2        2
   .\FRED.TXT       1       2        2
   C:\DOS\          2       1        .DAT
   C:\FRED.TXT      2       2        2              
   C:\DOS\FRED.TXT  2       2        2
}
      {clear arrays}
      for counter := 0 to 2 do Dir[counter,0] := #0;
      for counter := 0 to 2 do Name[counter,0] := #0;
      for counter := 0 to 2 do Ext[counter,0] := #0;
      FileSplit(HelpFileName,Dir[1],Name[1],Ext[1]);
      FileSplit(ParamList[0],Dir[2],Name[2],Ext[2]);

      if (StrPas(Dir[2]) = '.\') or (StrPas(Ext[2]) = '.') then
        Dir[0] := Dir[1]         {helpfile directory}
      else
        Dir[0] := Dir[2];        {specified directory, Windows if none}

      if StrPas(Name[2]) = '' then
        Name[0] := Name[1]       {helpfile name}
      else
        Name[0] := Name[2];      {specified name}

      if (StrPas(Ext[2]) = '.') or (StrPas(Ext[2]) = '') then
        StrCopy(Ext[0], '.dat')  {default extension}
      else
        Ext[0] := Ext[2];        {specified extension}


      StrECopy(StrECopy(StrECopy(DataFileName,Dir[0]),Name[0]),Ext[0]);
{      messagebox(hWindow,DataFileName,'Data file',8192);      {TEST}
      S := StrPas(ParamList[1]);
      if (S = '') then
        begin
        StrCopy(ParamList[1],'Author error - no Section name!!');         
        ParamNo := 1;
        goto NoParams;
        end;

      {convert ParamList[1], source of Nsect, to uppercase as windowheader}
      for counter := 0 to StrLen(ParamList[1]) do
        ParamList[1,counter] := UpCase(ParamList[1,counter]);
      NSect := ParamList[1];

      {no keyname provided, all keynames to be listed}
      if pos('=',StrPas(ParamList[2])) = 1 then                 
        begin
        Allkeys := True;
{        for counter := 0 to 255 do ParamList[2,counter] := #0;   {TEST}
        StrCopy(ParamList[2],'');                 
      end;

      {if no keyname provided, get and parse whole section}
      if StrLen(ParamList[2]) = 0 then
        begin
        for counter := 0 to 255 do ParamList[0,counter] := #0;   {TEST}
        for counter := 2 to MaxParam do                  {TEST}
          StrCopy(ParamList[counter],'');                {TEST}
          GetPrivateProfileString(Nsect, nil, '', ParamList[0], 255, DataFileName);

          ParamNo := 2;
          CharPos := 0;
          Separator := #0;
          For counter := 0 to 255 do
            begin
              if ParamList[0,counter] = Separator then {end of keyname}
                begin
                  ParamList[ParamNo,CharPos] := #0;   {terminate the string}
                  Inc(ParamNo);                       {ready for next keyname...}
                  CharPos := 0;                       {...its leading character}
                end
              else        {looking at a character in a keyname}
                begin
                  ParamList[ParamNo,CharPos] := ParamList[0,counter];
                  Inc(Charpos);
                end;
              if ParamNo = MaxParam then counter := 255;
            end;
          ParamList[ParamNo,CharPos] := #0;           {terminate final keyname}
          {only use real keynames}
          counter := MaxParam;
          while ParamList[counter,0] = #0 do dec(counter);
          ParamNo := counter;
        end;

      {get values for each keyname}
      for counter := 2 to ParamNo do
        begin
          Nkey := ParamList[counter];
          StrCopy(ParamList[counter],GetINIdata);
        end;

NoParams:       {jump here if essential parameters missing}

      { give the window a border }
      SetWindowLong (hWindow, GWL_STYLE,
                     GetWindowLong (hWindow, GWL_STYLE) or WS_BORDER);
      ewDatProc := 0;
    End;

    WM_PAINT :
    Begin           { paint the window }
      BeginPaint (hWindow, ps);
      DrawWindowText (ps.hdc);
      EndPaint (hWindow, ps);
      ewDatProc := 0;
    End;

    EWM_RENDER :
    Begin
    {
      Process the render message from WinHelp.
      wParam is the type of rendering requested.
      lParam points to a RENDERINFO structure.
    }
      Case wParam Of
        CF_BITMAP :
        Begin
    	    RenderInfo := pRenderInfo (lParam);
          dc := CreateCompatibleDC (0);
          if dc <> 0 Then
            Begin
              hfont := 0;

              { get window size }
              SendMessage (hWindow, EWM_QUERYSIZE, dc,
                           Longint (@pt));

              rc.left := 0;
              rc.top := 0;
              rc.right := pt.x;
              rc.bottom := pt.y;

              hbm := CreateCompatibleBitmap (RenderInfo^.dc,
                                             pt.x, pt.y);
              If hbm <> 0 Then
                Begin
                  hbmDefault := SelectObject (dc, hbm);
                  hbr := CreateSolidBrush (GetBkColor (dc));
                  If hbr <> 0 Then
                    Begin
                      FillRect (dc, rc, hbr);
                      DeleteObject (hbr);
                    End;

                  { Draw the border }
                  Rectangle (dc, rc.left, rc.top,
                             rc.right, rc.bottom);

                  { and then draw the text in the window }
                  DrawWindowText (dc);

                  hbm := SelectObject (dc, hbmDefault);
                End
              else        { not enough memory }
                hbm := 0;

            DeleteDC (dc);
	        End;

          ewDatProc := hbm;
        End;

        CF_TEXT :
        Begin
        {
          Create the text of the window for copying to
          the clipboard.  If you have bitmaps in the window,
          they will have to be omitted or represented by
          text characters.
          Return a handle to block of memory that contains
          the window's text.  WinHelp will free this memory
          when finished with it.
        }
	        gh := GlobalAlloc (GMEM_MOVEABLE or GMEM_NOT_BANKED, 512);

          If gh <> 0 Then
            Begin
              sz := GlobalLock (gh);
              StrCat (StrCopy (sz, ParamList[1]), ^M^J);
              for counter:=2 to ParamNo do
                StrCat (StrCat (sz, ParamList[counter]), ^M^J);

              GlobalUnlock (gh);
            End;

          ewDatProc := gh;
        End;

        { No other rendering types are supported }
        Else
          ewDatProc := 0;
      End;
    End;


    EWM_QUERYSIZE :
      Begin
      {
        Size query message.
        wParam is the target display context handle
        lParam points to a POINT structure in which the
        window's size should be returned.
        Return non-zero to indicate that we responded
        to this message.
      }
        hfont := SelectObject (wParam,GetStockObject (SYSTEM_FONT));
         
        GetTextMetrics (wParam, tm);
        lp := PPoint (lParam);

        {
          Calculate the horizontal (x) size by determining
          the length of the longest line of information.
        }  
        lp^.x := 0;

        for counter:= 1 to ParamNo do
          lp^.x := Max (lp^.x, LOWORD (
                        GetTextExtent (wParam, ParamList[counter],
                                       StrLen (ParamList[counter]))));

        lp^.x := lp^.x + 2*tm.tmMaxCharWidth;

        { Height is ParamNo plus a blank line
          before & after plus the size of the border. }
        lp^.y := ((ParamNo+2)*tm.tmHeight) + tm.tmExternalLeading;

        If hfont <> 0 Then
          SelectObject (wParam, hfont);
          ewDatProc := 1;
        End;

      {
        All other messages are processed by
        the default window procedure.
      }
      Else
        ewDatProc := DefWindowProc (hWindow, wMsg,
                                    wParam, lParam);
  End;
End;


{ InitEmbeddedWindow -- Register embedded window class }
Function InitEmbeddedWindow (hModule : THandle) : Boolean;
Var
  wc : TWndClass;
Begin
  wc.lpszClassName := 'ewDat';
  wc.style := CS_GLOBALCLASS;
  wc.hCursor := LoadCursor (0, IDC_ARROW);
  wc.hIcon := 0;
  wc.lpszMenuName := Nil;
  wc.hbrBackground := GetStockObject (WHITE_BRUSH);
  wc.hInstance := hModule;
  wc.lpfnWndProc := @ewDatProc;
  wc.cbClsExtra := 0;
  wc.cbWndExtra := 0;
  InitEmbeddedWindow := RegisterClass (wc);
End;


{ PtrString -- Build a pointer string in the form ssss:oooo }
Function PtrString (p : Longint) : String;
Const
  HexChar : Array [0..$F] of Char = '0123456789ABCDEF';
Var
  x : Integer;
  s : String[9];
Begin
  s := '0000:0000';
  For x := 1 to 9 Do
    Begin
      If x <> 5 Then
        Begin
          s[10-x] := HexChar[p and $0F];
          p := p div 16;
        End;
    End;
  PtrString := s;
End;


{ LDLLHandler -- Respond to WinHelp messages }
Function LDLLHandler (wMsg : Word; lParam1,
                      lParam2 : Longint) : Longint; Export;
Var
  s : String[20];

Begin
  case wMsg Of
    DW_WHATMSG :
      { 
        Return value indicates the types of messages 
        that you want to be notified of. 
      }
      LDLLHandler := DC_Callbacks or DC_Activate or DC_Jump or
                   DC_MinMax or DC_InitTerm;

    { Get callback addresses }
    DW_CALLBACKS :
      LDLLHandler := GetCallbacks (pFPtrs (lParam1), lParam2);

    { Received when WinHelp gains or loses focus }
    DW_ACTIVATE :
      LDLLHandler := 1;

    { Received when user selects a jump }
    DW_STARTJUMP :
      LDLLHandler := 1;

    { Received after the jump has been executed }
    DW_ENDJUMP :
{*Begin                                                         }
{*Timeout;                                                      }
      LDLLHandler := 1;
{*end;                                                          }

    { Received when WinHelp loads a new file }
    DW_CHGFILE :
      LDLLHandler := 1;

    { Received when the window is minimized or maximized }
    DW_MINMAX :
      LDLLHandler := 1;

    { Received when the window is sized }
    DW_SIZE :
      LDLLHandler := 1;

    { Received when the DLL is initialized }
    DW_INIT :
{*Begin                                                        }
{*Timeout;                                                     }
      LDLLHandler := 1;
{*end;                                                         }

    { Received immediately before the DLL is unloaded }
    DW_TERM :
        LDLLHandler := 1;

    Else
      LDLLHandler := 1;
  End;
End;

Exports
  ewDatProc,
  LDLLHandler;

Begin
  { Initialize the embedded window }
  If Not InitEmbeddedWindow (hInstance) Then
    ExitCode := 0;
End.

